#
# This program gives a combined electronic diary and time clock.
# Copyright John Lines (john@paladin.demon.co.uk) August 2001
#
# This program is released under the terms of the GNU Public Licence
#

package provide taglog_project 0.1

proc isbreak { projname } {
# return true if the given project is a break project
global projects

foreach proj $projects {
if { [lindex $proj 0] == $projname } {
  if { [lindex $proj 4] == 1 } {
	return 1
	} else {
	return 0
	}
  }
}
return 0
}

proc isoverhead { projname } {
# return true if the give project is an overhead project
global projects

foreach proj $projects {
if { [lindex $proj 0] == $projname } {
  if { [lindex $proj 3] == 1 } {
	return 1
	} else {
	return 0
	}
  }
}
return 0
}

proc isactive { projname } {
# return true if the give project is an overhead project
global projects

foreach proj $projects {
if { [lindex $proj 0] == $projname } {
  if { [lindex $proj 7] == 1 } {
	return 1
	} else {
	return 0
	}
  }
}
return 0
}


proc getprojcode { projname } {
# returns the project code asociated with the give project
global projects

foreach proj $projects {
if { [lindex $proj 0] == $projname } {
  return [lindex $proj 2]
  }
}
return ""
}

proc getprojstart { projname } {
global projects
foreach proj $projects {
if { [lindex $proj 0] == $projname } {
   return [lindex $proj 5]
  }
}
return ""
}

proc getprojend { projname } {
global projects
foreach proj $projects {
if { [lindex $proj 0] == $projname } {
   return [lindex $proj 6]
   }
}
return ""
}

proc projclosed { projname } {
global projects
 set enddate [getprojend $projname]
if { $enddate != "" } {
 # check the date - there is probably some clever way of doing this with the
 # tcl clock command but they should just compare as strings
 set now [clock format [clock seconds] -format "%Y-%m-%d"]
 if { [string compare $enddate $now] <=0 } {
   return 1
   }
  }
  return 0
} 

proc incprojactindex { projname } {
global projects
# increment the current actions index for a project and return the new value 
set n 0
foreach proj $projects {
if { [lindex $proj 0] == $projname } {
   set t [lindex $proj 1]
   if { $t == "" } {
	set t 0
	}
   incr t
   set proj [lreplace $proj 1 1 $t]
   set projects [lreplace $projects $n $n $proj]
   writeprojects
   return $t
   }
 incr n
}
return ""
}



proc getProjTimes { files } {
# Read the given files and return a list of  arrays giving the project times for# all the projects, with an extra array (first element ?) with the totals.

set a(unknown) "00:00"
set tot(unknown) "00:00"
set results {}

set index 0
foreach filename $files {
incr index
# FRINK: nocheck
set a$index(unknown) "00:00"

if [file exists $filename] {
# FRINK: nocheck
set tot$index "00:00"
set entries [tag readfile $filename]

foreach entry $entries {
 set starttime 0
 set endtime 0
 set thisproject "unknown"
# The entry should be a list of tag-value pairs
  foreach item $entry {
  set tagname [lindex $item 0]
  set tagvalue [lindex $item 1]
  if { $tagname == "End" } {
  # Because this is the end tag we should have all the values we are going
  # to get
  set duration [timediff $starttime $endtime]
  if { [info exists a($thisproject)] } {
      inctime a($thisproject) $duration
     } else {
      set a($thisproject) "00:00"
      inctime a($thisproject) $duration
     }
  } elseif { $tagname == "StartTime" } {
   set starttime $tagvalue
  } elseif { $tagname == "EndTime" } {
   set endtime $tagvalue
  } elseif { $tagname == "Project" } {
    set thisproject $tagvalue
  } else {
 }
}

}
} else {
 # we cant read the file
#FRINK: nocheck
 set tot$index "-"


}

}

return $results
}


proc addProjectOK {} {
global addproject_name addproject_breaks addproject_overheads addproject_bookas addproject_active projects addproject_startdate addproject_enddate

set projects_entry [list $addproject_name 1 $addproject_bookas $addproject_overheads $addproject_breaks $addproject_startdate $addproject_enddate $addproject_active]
lappend projects $projects_entry
# Also need to manually add a new projects menu entry
 .currentbar.project.menu add command -label $addproject_name \
	-command "set currentProject \"$addproject_name\""

# save preferences - though we may end up doing this on exit instead
# savePrefs
 writeprojects
destroy .addProject
}

proc doAddProject {} {
global addproject_overheads addproject_breaks addproject_bookas projects addproject_startdate addproject_enddate addproject_active

toplevel .addProject
wm title .addProject [mc "Add a new project"]

frame .addProject.entry
label .addProject.entry.l -text [mc "Project name"]
entry .addProject.entry.e -width 20 -textvariable addproject_name
pack .addProject.entry.l .addProject.entry.e -side left -in .addProject.entry
pack .addProject.entry

frame .addProject.flags
set addproject_breaks 0
checkbutton .addProject.flags.breaks -text [mc "Book as breaks"] -variable addproject_breaks
set addproject_overheads 0
checkbutton .addProject.flags.overheads -text [mc "Book as overheads"] -variable addproject_overheads
set addproject_active 1
checkbutton .addProject.flags.active -text [mc "Project is active"] -variable addproject_active
pack .addProject.flags.breaks .addProject.flags.overheads .addProject.flags.active -in .addProject.flags
pack .addProject.flags

frame .addProject.projcode
label .addProject.projcode.project -text [mc "Booking Code"]
#menubutton .addProject.projcode.project -text "Booking Code" -menu .addProject.bookas.project.m
#menu .addProject.bookas.project.m
#for { set i 0 } {$i < [llength $projects]} {incr i} 
# .addProject.bookas.project.m add command -label [lindex [lindex $projects $i] 0] \
#	-command "set addproject_bookas \"[lindex [lindex $projects $i] 0]\""
entry .addProject.projcode.projentry -width 20 -textvariable addproject_bookas
pack .addProject.projcode.project .addProject.projcode.projentry -in .addProject.projcode -side left
pack .addProject.projcode

set addproject_startdate [clock format [clock seconds] -format "%Y-%m-%d"]
frame .addProject.start
label .addProject.start.label -text [mc "Start Date"]
calUtil_win .addProject.start.e "" addproject_startdate
pack .addProject.start.label .addProject.start.e -in .addProject.start -side left
pack .addProject.start

frame .addProject.end
label .addProject.end.label -text [mc "End Date"]
calUtil_win .addProject.end.e "" addproject_enddate
pack .addProject.end.label .addProject.end.e -in .addProject.end -side left
pack .addProject.end

frame .addProject.bot
button .addProject.bot.ok -text OK -command addProjectOK
button .addProject.bot.cancel -text [mc Cancel] -command {doCancel .addProject }
button .addProject.bot.help -text [mc Help] -command "taghelp addproject"
pack .addProject.bot.ok .addProject.bot.cancel .addProject.bot.help -in .addProject.bot -side left
pack .addProject.bot

tkwait window .addProject
}

proc doEditProjectsOK {} {
global projects
global epbreaks epoverheads epbookas epdelete epstart epend epactive

set n 0
set projindex 0
foreach proj $projects {

if {$epdelete($n)} {
 set projects [lreplace $projects $projindex $projindex]
 incr n
} else {

# Insure against old project files with less elements
 while  {[ llength $proj] <8 } {
   lappend proj ""
 }
 

 set proj [lreplace $proj 4 4 $epbreaks($n) ]
 set proj [lreplace $proj 3 3 $epoverheads($n) ]
 set proj [lreplace $proj 2 2 $epbookas($n)]
 set proj [lreplace $proj 5 5 $epstart($n)]
 set proj [lreplace $proj 6 6 $epend($n)]
 set proj [lreplace $proj 7 7 $epactive($n)]
 set projects [lreplace $projects $projindex $projindex $proj]

incr n
incr projindex
}
}

writeprojects
destroy .ep
}

proc doEditProjects {} {
global projects
global epbreaks epoverheads epbookas epdelete epstart epend epactive

toplevel .ep
wm title .ep [mc "Edit Projects"]
wm minsize .ep 200 200

frame .ep.top
label .ep.top.title -width 20 -text [mc Title]

frame .ep.top.flags
label .ep.top.flags.breaks -text [mc Breaks]
label .ep.top.flags.overheads -text [mc Overheads]
label .ep.top.flags.active -text [mc Active]
pack .ep.top.flags.breaks  -in .ep.top.flags -anchor w
pack .ep.top.flags.overheads -in .ep.top.flags 
pack .ep.top.flags.active -in .ep.top.flags -anchor e

label .ep.top.code -text [mc Code]
label .ep.top.dates -text [mc Dates] -width 45
label .ep.top.delete -text [mc Delete]

pack .ep.top.title .ep.top.flags .ep.top.code .ep.top.dates .ep.top.delete -in .ep.top -side left
pack .ep.top


frame .ep.m -bd 4 -relief sunken
canvas .ep.m.c -yscrollcommand ".ep.m.scroll set" -scrollregion {0 0 0 650} -relief raised -confine false -yscrollincrement 25
scrollbar .ep.m.scroll -command ".ep.m.c yview" -relief raised

pack .ep.m.scroll -side right -fill y
pack .ep.m.c -side left -fill both -expand true

pack .ep.m -fill both -expand 1

set f [frame .ep.m.c.f -bd 0]
.ep.m.c create window 0 0 -anchor nw -window $f



set n 0
foreach proj $projects {

frame .ep.m.c.f.p$n
label .ep.m.c.f.p$n.p -text [lindex $proj 0] -width 20
set epbreaks($n) 0
if { [isbreak [lindex $proj 0]] } {
        set epbreaks($n) 1
        }
#checkbutton .ep.m.c.f.p$n.breaks -text [mc "Breaks"] -variable epbreaks($n) 
checkbutton .ep.m.c.f.p$n.breaks -variable epbreaks($n) 

pack .ep.m.c.f.p$n.p -side left
pack .ep.m.c.f.p$n.breaks -side left -fill x

set epoverheads($n) 0
if { [isoverhead [lindex $proj 0]] } {
	set epoverheads($n) 1
}
#checkbutton .ep.m.c.f.p$n.overheads -text "Overheads" -variable epoverheads($n)
checkbutton .ep.m.c.f.p$n.overheads -variable epoverheads($n)
pack .ep.m.c.f.p$n.overheads -side left -fill x

set epactive($n) 0
if {[isactive [lindex $proj 0]] } {
  set epactive($n) 1
}
checkbutton .ep.m.c.f.p$n.active -variable epactive($n)
pack .ep.m.c.f.p$n.active -side left -fill x

set epbookas($n) [getprojcode [lindex $proj 0]]
label .ep.m.c.f.p$n.bookasmenu -text "Code"
pack .ep.m.c.f.p$n.bookasmenu -side left -fill x
entry .ep.m.c.f.p$n.bookas -width 10 -textvariable epbookas($n)
pack .ep.m.c.f.p$n.bookas -side left -fill x


set epstart($n) [getprojstart [lindex $proj 0]]
menubutton .ep.m.c.f.p$n.startlab -text "Start" -menu .ep.m.c.f.p$n.startlab.m
menu .ep.m.c.f.p$n.startlab.m
calUtil_menu .ep.m.c.f.p$n.startlab.m .ep.m.c.f.p$n.starten
pack .ep.m.c.f.p$n.startlab -side left -fill x
entry .ep.m.c.f.p$n.starten -width 10 -textvariable epstart($n)
pack .ep.m.c.f.p$n.starten -side left -fill x

set epend($n) [getprojend [lindex $proj 0]]
menubutton .ep.m.c.f.p$n.endlab -text [mc "End "] -menu .ep.m.c.f.p$n.endlab.m
menu .ep.m.c.f.p$n.endlab.m
set todayiso [clock format [clock seconds] -format "%Y-%m-%d"]
set todaydisp [dateiso2disp $todayiso]
.ep.m.c.f.p$n.endlab.m add command -label "[mc Today] ($todaydisp)" -command "set epend($n) $todayiso"
calUtil_menu .ep.m.c.f.p$n.endlab.m .ep.m.c.f.p$n.enden
.ep.m.c.f.p$n.endlab.m add separator
.ep.m.c.f.p$n.endlab.m add command -label [mc Help] -command "taghelp ep_enddate"
pack .ep.m.c.f.p$n.endlab -side left -fill x
entry .ep.m.c.f.p$n.enden -width 10 -textvariable epend($n)
pack .ep.m.c.f.p$n.enden -side left -fill x


set epdelete($n) 0
checkbutton .ep.m.c.f.p$n.delete -text [mc "Delete"] -variable epdelete($n)
pack .ep.m.c.f.p$n.delete -side left -fill x
pack .ep.m.c.f.p$n -side top -fill x

incr n
}

set child [lindex [pack slaves $f] 0]

if { $child != ""} {
tkwait visibility $child
set incr [winfo height $child] 
    set width [winfo width $f]
    set height [winfo height $f]
 .ep.m.c config -scrollregion "0 0 $width $height"
 .ep.m.c config -yscrollincrement $incr
  set epmaxrows 12
    if { $height > $epmaxrows * $incr } {
        set height [expr $epmaxrows * $incr]
} 
.ep.m.c config -width $width -height $height
}


frame .ep.bot
button .ep.bot.ok -text OK -command {doEditProjectsOK}
button .ep.bot.cancel -text [mc Cancel] -command {doCancel .ep}
button .ep.bot.help -text [mc Help] -command "taghelp editproject"
pack .ep.bot.ok .ep.bot.cancel .ep.bot.help -in .ep.bot -side left
pack .ep.bot

tkwait window .ep

}

proc initProjTimes {} {
global projTimes projTimesTotal projTimesTotalNonBreaks
global projects

set projTimesTotal "00:00"
set projTimesTotalNonBreaks "00:00"

# initialise the projTimes array
foreach proj $projects {
 set projname [lindex $proj 0]
 if {[isactive $projname] } {
   if { ![info exists projTimes($projname)] } {
     set projTimes($projname) "00:00"
     }
   }
}

}

proc highlightActiveProjButton { varname index op } {
global projTimes currentProject

for { set i 1 } { $i <= [array size projTimes] } { incr i } {
 if { [winfo exists .projwin.$i.b] } {
 set bn [.projwin.$i.b cget -text]
# puts "bn $i is $bn"
 if { $bn == $currentProject } {
   .projwin.$i.b configure -background yellow
   } else {
   .projwin.$i.b configure -background grey
   }
 }
 }


}

proc doShowProjects {} {
global projTimes projTimesTotal projTimesTotalNonBreaks currentProject

toplevel .projwin
wm title .projwin [mc "Project Times"]

initProjTimes

set projlabelwidth 20
set totallabelwidth [expr $projlabelwidth + 2 ]


 set i 1
foreach project [array names projTimes] {
 frame .projwin.$i
 set projTxt $project
 if {$projTxt == "unknown"} {set projTxt [mc unknown]}
button .projwin.$i.b -text "$projTxt" -width $projlabelwidth -command "donext \"\" \"$project\""
 entry .projwin.$i.e -width 10 -textvariable projTimes($project) -state disabled
 pack .projwin.$i.b .projwin.$i.e -in .projwin.$i -side left
 pack .projwin.$i
 set duration [timediff "00:00:00" $projTimes($project)]
 inctime projTimesTotal $duration
 if { ! [isbreak $project ]} {
     inctime projTimesTotalNonBreaks $duration
     }
 incr i
}

frame .projwin.projtimestotal
label .projwin.projtimestotal.b -text [mc "Total"] -width $totallabelwidth
entry .projwin.projtimestotal.e -textvariable projTimesTotal -state disabled -width 10
pack .projwin.projtimestotal.b .projwin.projtimestotal.e -in .projwin.projtimestotal -side left
pack .projwin.projtimestotal

frame .projwin.projtimestotalnonbreaks
label .projwin.projtimestotalnonbreaks.b -text [mc "Total (non breaks)"] -width $totallabelwidth
entry .projwin.projtimestotalnonbreaks.e -textvariable projTimesTotalNonBreaks -state disabled -width 10
pack .projwin.projtimestotalnonbreaks.b .projwin.projtimestotalnonbreaks.e -in .projwin.projtimestotalnonbreaks -side left
pack .projwin.projtimestotalnonbreaks


frame .projwin.bot
button .projwin.bot.ok -text OK -command {doCancel .projwin}
button .projwin.bot.help -text [mc Help] -command "taghelp viewproject"
pack .projwin.bot.ok .projwin.bot.help -in .projwin.bot -side left
pack .projwin.bot

highlightActiveProjButton currentProject index w

trace variable currentProject w highlightActiveProjButton

tkwait window .projwin

trace vdelete currentProject w highlightActiveProjButton


}

proc setupProjMenu { menuname varname projstat helpdest } {
global projects

$menuname delete 1 end

$menuname add command -label "--" -command "set $varname  \"\""
for { set i 0 } {$i < [llength $projects]} {incr i} {
 if { $projstat == "all" ||  ! [projclosed [lindex [lindex $projects $i] 0]] } {
 $menuname add command -label [lindex [lindex $projects $i] 0] \
	-command "set $varname \"[lindex [lindex $projects $i] 0]\""
}
}
$menuname add separator
$menuname add command -label [mc Help] -command "taghelp $helpdest"
}


proc writeprojects {} {
global projects projectsfilename
# for now this is more complex than writeactions because we have the projects
# in a tcl list which is not in tagged format.

set f [open $projectsfilename w]

puts $f "Tag-accountproj-version: 1.0"
puts $f "End:"
puts $f ""

foreach proj $projects {
set projname [lindex $proj 0]
puts $f "Name: $projname"
set projnextact [lindex $proj 1]
puts $f "NextActionId: $projnextact"
set projcode [lindex $proj 2]
puts $f "Code: $projcode"
if {[lindex $proj 3] == 1} {
 puts $f "Flags: Overheads"
}
if {[lindex $proj 4] == 1} {
 puts $f "Flags: Breaks"
}
if {[lindex $proj 7]== 1} {
 puts $f "Flags: Active"
}
set projstart [lindex $proj 5]
puts $f "StartDate: $projstart"
set projend [lindex $proj 6]
if { $projend != ""} {
puts $f "EndDate: $projend"
}

puts $f "End:"
}

close $f
}

proc readprojects {} {

global projects projectsfilename allproj
# Note that projects is not in tagged list format

set allproj [tag readfile $projectsfilename]
if { [ llength $allproj] == 0 } {
  set hdr ""
  set header [list Tag-accountproj-version 1.0]
  lappend hdr $header
  set endpair [list End ]
  lappend hdr $endpair
  lappend  allproj $hdr
  }
# puts "allproj is $allproj"

# projects may have been read in from preferences
# if they have then exit for now
if {[info exists projects] } {
# puts "projects is $projects"
 if { [llength $projects] != 0 } {
 writeprojects
 savePrefs
 return
 }
 unset projects
 }

# set projects to match allproj for transition
foreach entry $allproj {
 set thisproj_name ""
 set thisproj_actidx 0
 set thisproj_code ""
 set thisproj_isoverhead 0
 set thisproj_isbreak 0
 set thisproj_isactive 0
 set thisproj_start ""
 set thisproj_end ""
 foreach item $entry {
 if {[lindex $item 0 ] == "Name" } { set thisproj_name [lindex $item 1]}
 if {[lindex $item 0 ] == "NextActionId" } { set thisproj_actidx [lindex $item 1]}
 if {[lindex $item 0 ] == "Code" } { set thisproj_code [lindex $item 1]}
 if {[lindex $item 0 ] == "Flags" } {
   if { [string match Overheads [lindex $item 1]]} { set thisproj_isoverhead 1}
   if { [string match Breaks [lindex $item 1]]} { set thisproj_isbreak 1}
   if { [string match Active [lindex $item 1]]} { set thisproj_isactive 1}
   }
 if {[lindex $item 0 ] == "StartDate" } { set thisproj_start [lindex $item 1]}
 if {[lindex $item 0 ] == "EndDate" } { set thisproj_end [lindex $item 1]}
 }

 if { $thisproj_name != ""} {
 set thisproj [list $thisproj_name $thisproj_actidx $thisproj_code $thisproj_isoverhead $thisproj_isbreak $thisproj_start $thisproj_end $thisproj_isactive]
 lappend projects $thisproj
 }
 
 }

# puts "projects is $projects"
# If we have no projects yet we will create an empty list
if { ![info exists projects] } { set projects {} }

}
